# coding: utf8

"""T1 Linear - Clinica Visualizer.
This file has been generated automatically by the `clinica generate template`
command line tool. See here for more details:
http://clinica.run/doc/InteractingWithClinica/
"""
